package com.chthonics.vaadin.issues;

import com.vaadin.ui.Button;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
import org.vaadin.addon.leaflet.LCircleMarker;
import org.vaadin.addon.leaflet.LFeatureGroup;
import org.vaadin.addon.leaflet.LMap;
import org.vaadin.addon.leaflet.LPolyline;
import org.vaadin.addon.leaflet.LTileLayer;
import org.vaadin.addon.leaflet.LeafletClickEvent;
import org.vaadin.addon.leaflet.LeafletClickListener;
import org.vaadin.addon.leaflet.editable.FeatureModifiedEvent;
import org.vaadin.addon.leaflet.editable.FeatureModifiedListener;
import org.vaadin.addon.leaflet.editable.LEditable;
import org.vaadin.addon.leaflet.editable.LEditableMap;
import org.vaadin.addon.leaflet.shared.Point;

/**
 *
 * @author Warwick DUfour
 */
public class LeafletEditableMapComponent2 extends CustomComponent {

	private LMap lMap;
	private LEditable lPolylineEditable;
	private LEditableMap lEditableMap;
	private boolean polylineIsEditable = false;
	private LFeatureGroup lCircleMarkerGroup;
	LPolyline lPolyline;

	public LeafletEditableMapComponent2() {

		lMap = new LMap();
		lMap.setCustomInitOption("editable", true);
		lMap.setHeight("300px");
		lMap.setCenter(0, 0);
		lMap.setZoomLevel(0);
		lMap.addLayer(new LTileLayer("http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"));

		lPolyline = new LPolyline(new Point(40, -10), new Point(70, 0), new Point(40, 10));
		lPolyline.setId("polyline1");
		lPolyline.setImmediate(true);
		
		// Create a group of circle markers to reflect vertices of the polyline
		lCircleMarkerGroup = new LFeatureGroup();
		lCircleMarkerGroup.setImmediate(true);
		int count = 0;
		for (Point point : lPolyline.getPoints()) {
			LCircleMarker lCircleMarker = createLCircleMarker(point, count);
			lCircleMarkerGroup.addComponent(lCircleMarker);
			count++;
		}

		// Add polyline and marker to map
		lMap.addComponents(lPolyline, lCircleMarkerGroup);
		lMap.addClickListener(new LeafletClickListener() {

			@Override
			public void onClick(LeafletClickEvent event) {
				System.out.println("lMap Clicked");
			}
		});
		
		// Create a row of buttons to be positioned under the map
		Button button1 = new Button("Start Editing", new Button.ClickListener() {

			@Override
			public void buttonClick(Button.ClickEvent event) {
				System.out.println("Button 1 clicked");
				if(!polylineIsEditable) {
					lPolylineEditable = new LEditable(lPolyline);
					addlPolylineEditableFeatureModifiedListener();
					polylineIsEditable = true;
				}

			}

		});
		Button button2 = new Button("Stop Editing", new Button.ClickListener() {

			@Override
			public void buttonClick(Button.ClickEvent event) {
				System.out.println("Button 2 clicked");
				if(polylineIsEditable) {
					lPolylineEditable.remove();
					lCircleMarkerGroup.removeAllComponents();
					int count = 0;
					for (Point point : lPolyline.getPoints()) {
						LCircleMarker lCircleMarker = createLCircleMarker(point, count);
						lCircleMarkerGroup.addComponent(lCircleMarker);
						count++;
					}
					lCircleMarkerGroup.markAsDirty();
					polylineIsEditable = false;
				}
			}
		});
		Button button3 = new Button("Print Status", new Button.ClickListener() {

			@Override
			public void buttonClick(Button.ClickEvent event) {
				System.out.println("Button 3 clicked");
				System.out.println("Polyline editable? " + polylineIsEditable);
				System.out.println("Polyline Points:");
				int count = 0;
				for (Point point : lPolyline.getPoints()) {
					System.out.println("|   Point " + count + " position = " + point);
					count++;
				}
			}
		});
		HorizontalLayout horizontalLayout = new HorizontalLayout();
		horizontalLayout.setMargin(true); // Very useful
		horizontalLayout.setId("button-panel-layout");
		horizontalLayout.addComponents(button1, button2, button3);

		// A layout structure used for composition
		Panel panel = new Panel("Editable Leaflet Map Component 2");
		VerticalLayout verticalLayout = new VerticalLayout();
		verticalLayout.setId("main-panel-layout");
		panel.setContent(verticalLayout);

		// Set the size as undefined at all levels
		panel.setSizeFull();
		setSizeFull();
		// panelContent.setSizeFull();

		/* Build layout */
		verticalLayout.setMargin(true);
		verticalLayout.setSpacing(true);
		verticalLayout.addComponents(lMap, horizontalLayout);
		verticalLayout.setSizeFull();

		// The composition root MUST be set
		setCompositionRoot(panel);
	}

	private void addListeners(LCircleMarker lCircleMarker) {
		lCircleMarker.addClickListener(new LeafletClickListener() {

			@Override
			public void onClick(LeafletClickEvent event) {
				LCircleMarker lCircleMarker = (LCircleMarker) event.getSource();
				System.out.println("lCircleMarker " + ((LCircleMarker) event.getSource()).getId() + " clicked.");
				if(!polylineIsEditable) {
					System.out.println("    Opening popup.");
					lCircleMarker.openPopup();
				}
			}
		});
//		lCircleMarker.addMouseOutListener(new LeafletMouseOutListener() {
//
//			@Override
//			public void onMouseOut(LeafletMouseOutEvent event) {
//				System.out.println("lCircleMarker " + ((LCircleMarker) event.getSource()).getId() + " mouse out event.");
//			}
//		});
//		lCircleMarker.addMouseOverListener(new LeafletMouseOverListener() {
//
//			@Override
//			public void onMouseOver(LeafletMouseOverEvent event) {
//				LCircleMarker lCircleMarker = (LCircleMarker) event.getSource();
//				System.out.println("lCircleMarker " + ((LCircleMarker) event.getSource()).getId() + " mouse over event.");
//				if(!polylineIsEditable) {
//					System.out.println("    Opening popup.");
//					lCircleMarker.openPopup();
//				}
//			}
//		});
	}

	private LCircleMarker createLCircleMarker(Point point, int count) {
		LCircleMarker lCircleMarker = new LCircleMarker(point, 10);
		lCircleMarker.setImmediate(true);
		lCircleMarker.setPopup("lCircleMarker" + count + ", position = " + point.toString());
		addListeners(lCircleMarker);
		lCircleMarker.setId("lCircleMarker" + count);
		lCircleMarker.setColor("green");
		return lCircleMarker;
	}
	
	private void addlPolylineEditableFeatureModifiedListener() {
		
		lPolylineEditable.addFeatureModifiedListener(new FeatureModifiedListener() {

			@Override
			public void featureModified(FeatureModifiedEvent event) {
				System.out.println("LEditable FeatureModifiedEvent received.");
				LEditable lEditable = (LEditable) event.getSource();
				if(lEditable != lPolylineEditable) {
					throw new RuntimeException("Software Error");
				}
				lEditable.remove();
				lPolylineEditable = new LEditable(lPolyline);
				addlPolylineEditableFeatureModifiedListener();
			}
		});
	}
}
